#!/usr/bin/env python

"""
modeling.py
Supplementary data for:
'RNA tertiary structure prediction with ModeRNA'
by Magdalena Rother, Kristian Rother, Tomasz Puton, Janusz M. Bujnicki

This is a script for modeling the structure of tRNA(Thr) from E coli.

Neccesary input files:
 - 1EHZ.pdb
 - 1C0A.pdb
 - target_1EHZ.fasta
 - target_1C0A.fasta

Run: python modeling.py

http://genesilico.pl/moderna/
http://www.python.org/
"""

from moderna import *


###################
# INPUT ANALYSIS  #
###################

# LOAD INPUT DATA
t = load_template('1EHZ.pdb') 
a = load_alignment('target_1EHZ.fasta')
t2 = load_template('1C0A.pdb','B')
a2 = load_alignment('target_1C0A.fasta')

# ANALYZE INPUT DATA FOR 1EHZ
print '1EHZ sequence:\n', get_sequence(t)
print '\n1EHZ problem analysis:\n', examine_structure(t)
print '\nCleaning structure:', clean_structure(t)
print '\n1EHZ secondary structure:\n', get_secstruc(t)
print '\n1EHZ geometry analysis:', analyze_geometry(t), find_clashes(t)
print 'Matching template with alignment:', match_template_with_alignment(t,a)

# ANALYZE INPUT DATA FOR 1C0A
print '1EHZ sequence:\n', get_sequence(t2)
print '\n1EHZ problem analysis:\n', examine_structure(t2)
print '\nCleaning structure:', clean_structure(t2)
print '\n1C0A secondary structure:\n', get_secstruc(t2)
print '\n1C0A geometry analysis:', analyze_geometry(t2), find_clashes(t2)
print 'Matching template with alignment:', match_template_with_alignment(t2,a2)



###################################
# MODIFICATION EXEMPLARY ANALYSIS #
###################################

# 1EHZ MODIFICATION ANALYSIS
print '\n\n______________________________________________________'
print 'Exemplary analysis of modifications:\n\n'

print 'Modifications present in 1EHZ:\n',find_modifications(t)
print '\nDifferent name representations of residue 34'
print 'Full name: ',t['34'].full_name
print 'Long abbreviation: ', t['34'].long_abbrev
print 'One letter abbreviation: ', t['34'].short_abbrev
print 'New abbreviation (numeric code): ', t['34'].new_abbrev
print 'PDB abbreviation: ', t['34'].pdb_abbrev
print '\nDifferent representation of modification in the sequence:'
print get_sequence(t)
print get_sequence(t).seq_new_notation



###################
# MODELING CASE 1 #
###################
print '\n\n______________________________________________________'
print 'Modeling of E. coli tRNA(Thr) on 1EHZ\n\n'

# LOAD INPUT DATA
t = load_template('1EHZ.pdb') 
clean_structure(t)
a = load_alignment('target_1EHZ.fasta')

# BUILD MODEL
m_1EHZ = create_model(t,a)
write_model(m_1EHZ, 'model_1EHZ.pdb')

# EVALUATE MODEL
print 'Model sequence:\n',get_sequence(m_1EHZ)
print 'Matching model sequence with target sequence:', match_alignment_with_model(a,m_1EHZ)
print 'Model geometry analysis:', analyze_geometry(m_1EHZ)
print 'Clashes present in model:', find_clashes(m_1EHZ)



###################
# MODELING CASE 2 #
###################
print '\n\n______________________________________________________'
print 'Modeling of E. coli tRNA(Thr) on 1C0A\n\n'

# LOAD INPUT DATA
t2 = load_template('1C0A.pdb','B')
clean_structure(t2)
a2 = load_alignment('target_1C0A.fasta')

# BUILD MODEL
m_1C0A = create_model(t2, a2)
write_model(m_1C0A, 'model_1C0A.pdb')

# EVALUATE MODEL
print 'Model sequence:\n',get_sequence(m_1C0A)
print 'Matching model sequence with target sequence:', match_alignment_with_model(a,m_1C0A)
print 'Model geometry analysis:', analyze_geometry(m_1C0A)
print 'Clashes present in model:' ,find_clashes(m_1C0A)



###################
# MODELING CASE 3 #
###################
print '\n\n______________________________________________________'
print 'Modeling of E. coli tRNA(Thr) on 1EHZ + 9nt insertion\n\n'

# LOAD INPUT DATA
t = load_template('1EHZ.pdb') 
clean_structure(t)
a = load_alignment('target_1EHZ.fasta')
st_1C0A = load_model('1C0A.pdb','B')

# PREPARE FRAGMENT FOR INSERTION
for resi in st_1C0A:
    if resi.identifier not in ['630','631','632','633','634','635','636','637','638','639','640']:
        delete_residue(resi.identifier, st_1C0A)
write_model(st_1C0A, 'anticodon_fragment_9nt.pdb') # the shortest fragment (9 + 2 anchors)

# BUILD MODEL
m_1EHZ_9nt = create_model(t, a)
anticodon_9nt = create_fragment('anticodon_fragment_9nt.pdb', m_1EHZ_9nt['30'], m_1EHZ_9nt['40'], chain_name='B', sequence='AUUCGUEAU')
insert_fragment(m_1EHZ_9nt, anticodon_9nt)
write_model(m_1EHZ_9nt, 'model_1EHZ_insertion_9nt.pdb')

# EVALUATE MODEL
print 'Model sequence:\n',get_sequence(m_1EHZ_9nt)
print 'Matching model sequence with target sequence:', match_alignment_with_model(a,m_1EHZ_9nt)
print 'Model geometry analysis:', analyze_geometry(m_1EHZ_9nt)
print 'Clashes present in model:', find_clashes(m_1EHZ_9nt)



###################
# MODELING CASE 4 #
###################
print '\n\n______________________________________________________'
print 'Modeling of E. coli tRNA(Thr) on 1EHZ + 15nt insertion\n\n'

# LOAD INPUT DATA
t = load_template('1EHZ.pdb') 
clean_structure(t)
a = load_alignment('target_1EHZ.fasta')
st_1C0A = load_model('1C0A.pdb','B')

# PREPARE FRAGMENT FOR INSERTION
for resi in st_1C0A:
    if resi.identifier not in ['627','628','629','630','631','632','633','634','635','636','637','638','639','640','641','642','643']:
        delete_residue(resi.identifier, st_1C0A)
write_model(st_1C0A, 'anticodon_fragment_15nt.pdb') # medium fragment (15 + 2 anchors)

# BUILD MODEL
m_1EHZ_15nt = create_model(t, a)
anticodon_15nt = create_fragment('anticodon_fragment_15nt.pdb', m_1EHZ_15nt['27'], m_1EHZ_15nt['43'], chain_name='B', sequence='CGCAUUCGUEAUGCG')
insert_fragment(m_1EHZ_15nt, anticodon_15nt)
write_model(m_1EHZ_15nt, 'model_1EHZ_insertion_15nt.pdb')

# EVALUATE MODEL
print 'Model sequence:\n',get_sequence(m_1EHZ_15nt)
print 'Matching model sequence with target sequence:', match_alignment_with_model(a,m_1EHZ_15nt)
print 'Model geometry analysis:', analyze_geometry(m_1EHZ_15nt)
print 'Clashes present in model:', find_clashes(m_1EHZ_15nt)



###################
# MODELING CASE 5 #
###################
print '\n\n______________________________________________________'
print 'Modeling of E. coli tRNA(Thr) on 1EHZ + 19nt insertion\n\n'

# LOAD INPUT DATA
t = load_template('1EHZ.pdb') 
clean_structure(t)
a = load_alignment('target_1EHZ.fasta')
st_1C0A = load_model('1C0A.pdb','B')

# PREPARE FRAGMENT FOR INSERTION
for resi in st_1C0A:
    if resi.identifier not in ['625','626','627','628','629','630','631','632','633','634','635','636','637','638','639','640','641','642','643','644','645']:
        delete_residue(resi.identifier, st_1C0A)
write_model(st_1C0A, 'anticodon_fragment_19nt.pdb') # medium fragment (19 + 2 anchors)

# BUILD MODEL
m_1EHZ_19nt = create_model(t, a)
anticodon_19nt = create_fragment('anticodon_fragment_19nt.pdb', m_1EHZ_19nt['25'], m_1EHZ_19nt['45'], chain_name='B', sequence='AGCGCAUUCGUEAUGCGAA')
insert_fragment(m_1EHZ_19nt, anticodon_19nt)
write_model(m_1EHZ_19nt, 'model_1EHZ_insertion_19nt.pdb')

# EVALUATE MODEL
print 'Model sequence:\n',get_sequence(m_1EHZ_19nt)
print 'Matching model sequence with target sequence:', match_alignment_with_model(a,m_1EHZ_19nt)
print 'Model geometry analysis:', analyze_geometry(m_1EHZ_19nt)
print 'Clashes present in model:', find_clashes(m_1EHZ_19nt)
